from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base

DatabasePath = str

Base = declarative_base()


class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonMeta,cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class SingletonClass(metaclass=SingletonMeta):
    pass


class Database(SingletonClass):
    def __init__(self, path: DatabasePath):
        self.__engine = create_engine(path)
        self.__session = sessionmaker(self.__engine, expire_on_commit=False, autoflush=False)

    def get_session(self):
        session = self.__session()

        try:
            yield session
        finally:
            session.close()

    def create_all(self):
        Base.metadata.create_all(self.__engine)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hackpi']

package_data = \
{'': ['*']}

install_requires = \
['fastapi>=0.89.1,<0.90.0',
 'pydantic>=1.10.4,<2.0.0',
 'sqlalchemy>=2.0.0,<3.0.0']

setup_kwargs = {
    'name': 'hackpi',
    'version': '0.1.0',
    'description': 'HackPi - is a library for the FastAPI framework that adds additional functionality that simplifies development.',
    'long_description': '',
    'author': 'thenesterov',
    'author_email': 'thenesterov@proton.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/thenesterov/hackpi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from fastapi import APIRouter, Depends
from pydantic import BaseModel

from hackpi.Database import Base, Database


class Router:
    def __init__(self, database: Database, model: Base, schema: BaseModel):
        self.__database = database
        self.__model = model
        self.__schema = schema

        self.__router = APIRouter(prefix=f'/{self.__model.__name__.lower()}', tags=[self.__model.__name__.lower()])

    def get_router(self):
        @self.__router.get(f'/{self.__model.__name__.lower()}')
        def get_records(session=Depends(self.__database.get_session)):
            return session.query(self.__model).all()

        @self.__router.get(f'/{self.__model.__name__.lower()}/' + 'id')
        def get_record(id: int, session=Depends(self.__database.get_session)):
            return session.query(self.__model).filter(self.__model.id == id).one()

        @self.__router.post(f'/{self.__model.__name__.lower()}')
        def set_record(schema: self.__schema, session=Depends(self.__database.get_session)):
            session.add(self.__model(
                **schema.__dict__
            ))
            session.commit()
            return 201

        @self.__router.put(f'/{self.__model.__name__.lower()}/' + 'id')
        def update_record(id: int, schema: self.__schema, session=Depends(self.__database.get_session)):
            session.query(self.__model).filter(self.__model.id == id).update(
                schema.__dict__
            )
            session.commit()
            return 200

        @self.__router.delete(f'/{self.__model.__name__.lower()}/' + 'id')
        def delete_record(id: int, session=Depends(self.__database.get_session)):
            session.query(self.__model).filter(self.__model.id == id).delete()
            session.commit()
            return 200

        return self.__router
